"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
const RoutingControllers = require("@benjd90/routing-controllers");
const n9_node_utils_1 = require("@neo9/n9-node-utils");
const express_1 = require("@promster/express");
const PromsterServer = require("@promster/server");
const Sentry = require("@sentry/node");
const ClassValidator = require("class-validator");
const express = require("express");
const fast_safe_stringify_1 = require("fast-safe-stringify");
const helmet_1 = require("helmet");
const http_1 = require("http");
const morgan = require("morgan");
const PrometheusClient = require("prom-client");
const typedi_1 = require("typedi");
const requestid_1 = require("./requestid");
const utils_1 = require("./utils");
async function init(options, packageJson, log, conf) {
    // Setup routing-controllers to use typedi container.
    RoutingControllers.useContainer(typedi_1.Container);
    ClassValidator.useContainer(typedi_1.Container);
    // Listeners
    const analyzeError = (error) => {
        /* istanbul ignore if */
        if (error.syscall !== 'listen') {
            return error;
        }
        // handle specific listen errors with friendly messages
        switch (error.code) {
            case 'EACCES':
                return new Error(`Port ${options.http.port} requires elevated privileges`);
            case 'EADDRINUSE':
                return new Error(`Port ${options.http.port} is already in use`);
            /* istanbul ignore next */
            default:
                return error;
        }
    };
    const onListening = () => {
        options.log.info(`Listening on port ${options.http.port}`);
        if (options.prometheus.isEnabled) {
            (0, express_1.signalIsUp)();
        }
    };
    // Create HTTP server
    let expressApp = express();
    let prometheusServer;
    if (options.sentry) {
        const initOptions = {
            release: packageJson.version,
            ...options.sentry.initOptions,
        };
        if (options.sentry.additionalIntegrations?.length) {
            initOptions.integrations = initOptions.integrations || [];
            for (const additionalIntegration of options.sentry.additionalIntegrations) {
                if (additionalIntegration.kind === 'tracing') {
                    initOptions.integrations.push(new Sentry.Integrations.Express({
                        // to trace all requests to the default router
                        app: expressApp,
                        methods: additionalIntegration.options?.methods,
                    }));
                }
            }
        }
        if (options.log.isLevelEnabled('debug')) {
            const initOptionsCopy = { ...initOptions };
            delete initOptionsCopy.dsn;
            options.log.debug(`Sentry conf`, initOptionsCopy);
        }
        Sentry.init(initOptions);
    }
    // Middleware
    expressApp.use(requestid_1.setRequestContext);
    expressApp.use((0, helmet_1.default)());
    if (options.prometheus.isEnabled) {
        expressApp.use((0, express_1.createMiddleware)({
            options: {
                normalizePath: (path, context) => context.req.route?.path || context.req.originalUrl || context.req.url,
                labels: options.prometheus.labels,
                getLabelValues: options.prometheus.getLabelValues,
                skip: options.prometheus.skip,
            },
        }));
        new PrometheusClient.Gauge({
            name: 'version_info',
            help: 'App version',
            labelNames: ['version', 'name'],
        }).set({ version: packageJson.version, name: packageJson.name }, 1);
        if (!(await (0, utils_1.isPortAvailable)(options.prometheus.port))) {
            throw new n9_node_utils_1.N9Error('prometheus-server-port-unavailable', 500, {
                port: options.prometheus.port,
            });
        }
        prometheusServer = await PromsterServer.createServer({ port: options.prometheus.port });
    }
    // Logger middleware
    if (options.http.logLevel) {
        expressApp.use(morgan(options.http.logLevel, {
            stream: {
                write: (message) => {
                    if (options.log.formatJSON) {
                        try {
                            const morganDetails = JSON.parse(message);
                            options.log.info(`api call ${morganDetails.path}`, morganDetails);
                        }
                        catch (e) {
                            options.log.info(message?.replace('\n', ''), { errString: (0, fast_safe_stringify_1.default)(e) });
                        }
                    }
                    else {
                        options.log.info(message?.replace('\n', ''));
                    }
                },
            },
        }));
    }
    const server = (0, http_1.createServer)(expressApp);
    if (options.http.beforeRoutingControllerLaunchHook) {
        await options.http.beforeRoutingControllerLaunchHook({
            expressApp,
            log: log.module('before-hook'),
            options,
            conf,
        });
    }
    expressApp.use(express.json(options.http.bodyParser));
    expressApp = RoutingControllers.useExpressServer(expressApp, options.http.routingController);
    if (options.http.afterRoutingControllerLaunchHook) {
        await options.http.afterRoutingControllerLaunchHook({
            expressApp,
            log: log.module('after-hook'),
            options,
            conf,
        });
    }
    // Listen method
    const listen = async () => {
        return new Promise((resolve, reject) => {
            server.listen(options.http.port);
            server.on('error', (error) => {
                reject(analyzeError(error));
            });
            server.on('listening', () => {
                onListening();
                resolve();
            });
        });
    };
    // Make the server listen
    if (!options.http.preventListen)
        await listen();
    return {
        server,
        prometheusServer,
        conf,
        app: expressApp,
        logger: log,
    };
}
exports.init = init;
//# sourceMappingURL=express-app.js.map