"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateConf = void 0;
const n9_node_utils_1 = require("@neo9/n9-node-utils");
const class_transformer_1 = require("class-transformer");
const class_validator_1 = require("class-validator");
function formatWhitelistErrors(validationErrors, prefix) {
    const formattedWhitelistErrors = [];
    for (const validationError of validationErrors) {
        const computedPrefix = prefix
            ? `${prefix}.${validationError.property}`
            : validationError.property;
        if (validationError.constraints?.whitelistValidation) {
            formattedWhitelistErrors.push([
                computedPrefix,
                validationError.constraints.whitelistValidation,
            ]);
        }
        if (validationError.children) {
            formattedWhitelistErrors.push(...formatWhitelistErrors(validationError.children, computedPrefix));
        }
    }
    return formattedWhitelistErrors;
}
function formatValidationErrors(validationErrors, prefix) {
    const formattedErrors = [];
    for (const validationError of validationErrors) {
        const computedPrefix = prefix
            ? `${prefix}.${validationError.property}`
            : validationError.property;
        if (validationError.constraints) {
            const formattedError = [computedPrefix, []];
            for (const key of Object.keys(validationError.constraints)) {
                formattedError[1].push(`${key}: ${validationError.constraints[key]}`);
            }
            formattedErrors.push(formattedError);
        }
        if (validationError.children) {
            formattedErrors.push(...formatValidationErrors(validationError.children, computedPrefix));
        }
    }
    return formattedErrors;
}
function assertClassTypeIsGiven(validationOptions) {
    if (!validationOptions.classType) {
        throw new n9_node_utils_1.N9Error('N9NodeRouting configuration validation options are not correct validation is enabled but' +
            ' no classType is given', 500, validationOptions);
    }
}
function handleValidationErrors(validationErrors, logger, options) {
    const noError = validationErrors.length === 0;
    if (noError)
        return;
    const validationErrorsWithoutExcludeProperties = (0, class_transformer_1.classToPlain)(validationErrors);
    if (!options.formatValidationErrors) {
        logger.error('Configuration is not valid', {
            validationErrors: JSON.stringify(validationErrorsWithoutExcludeProperties),
        });
        throw new n9_node_utils_1.N9Error('Configuration is not valid', 500, {
            validationErrors: validationErrorsWithoutExcludeProperties,
        });
    }
    logger.error('Configuration is not valid: ');
    const validationErrorsFormatted = formatValidationErrors(validationErrors, undefined);
    for (const validationError of validationErrorsFormatted) {
        const attributePath = validationError[0];
        const errorList = validationError[1];
        for (const error of errorList) {
            logger.error(`Attribute ${attributePath} is not valid - ${error}`);
        }
    }
    throw new n9_node_utils_1.N9Error('Configuration is not valid', 500, {
        validationErrors: validationErrorsWithoutExcludeProperties,
    });
}
function handleWhitelistErrors(whitelistErrors, logger, options) {
    const noError = whitelistErrors.length === 0;
    if (noError)
        return;
    const whitelistErrorsWithoutExcludeProperties = (0, class_transformer_1.classToPlain)(whitelistErrors);
    if (!options.formatWhitelistErrors) {
        logger.warn('Configuration contains unexpected attributes / Please remove those attributes', {
            warnings: JSON.stringify(whitelistErrorsWithoutExcludeProperties),
        });
        return;
    }
    logger.warn('Configuration contains unexpected attributes:');
    const whitelistErrorsFormatted = formatWhitelistErrors(whitelistErrors, undefined);
    for (const whitelistError of whitelistErrorsFormatted) {
        const attributePath = whitelistError[0];
        logger.warn(`Please remove attribute '${attributePath}' from the configuration`);
    }
}
async function validateConf(conf, validationOptions, logger) {
    const validationIsEnabled = validationOptions && validationOptions.isEnabled;
    if (!validationIsEnabled) {
        logger.info('Configuration validation is disabled');
        return;
    }
    assertClassTypeIsGiven(validationOptions);
    logger.info('Checking configuration...');
    const confInstance = (0, class_transformer_1.plainToClass)(validationOptions.classType, conf);
    const validationErrors = await (0, class_validator_1.validate)(confInstance);
    handleValidationErrors(validationErrors, logger, validationOptions);
    const whitelistErrors = await (0, class_validator_1.validate)(confInstance, {
        whitelist: true,
        forbidNonWhitelisted: true,
    });
    handleWhitelistErrors(whitelistErrors, logger, validationOptions);
    logger.info('Configuration is valid');
    return confInstance;
}
exports.validateConf = validateConf;
//# sourceMappingURL=init-conf.js.map