"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLoadingConfOptions = exports.mergeOptionsAndConf = exports.applyDefaultValuesOnOptions = exports.getModulesPath = exports.applyConfOptionsDefaults = void 0;
const n9_node_log_1 = require("@neo9/n9-node-log");
const n9_node_utils_1 = require("@neo9/n9-node-utils");
const Sentry = require("@sentry/node");
const appRootDir = require("app-root-dir");
const fs = require("fs");
const _ = require("lodash");
const morgan = require("morgan");
const path_1 = require("path");
const error_handler_interceptor_1 = require("./middleware/error-handler.interceptor");
const prometheus_interceptor_1 = require("./middleware/prometheus.interceptor");
const sentry_request_interceptor_1 = require("./middleware/sentry-request.interceptor");
const sentry_tracing_interceptor_1 = require("./middleware/sentry-tracing.interceptor");
const session_loader_interceptor_1 = require("./middleware/session-loader.interceptor");
const Utils = require("./utils");
const envsWithoutJSONLogs = ['development', 'test'];
function applyLogsOptionsDefaults(options, appName, environment) {
    // If enableLogFormatJSON is provided, we use it's value.
    // Otherwise, we activate the plain logs for development env and JSON logs for other environments
    options.enableLogFormatJSON =
        typeof options.enableLogFormatJSON === 'boolean'
            ? options.enableLogFormatJSON
            : !envsWithoutJSONLogs.includes(environment);
    if (!options.log) {
        options.log = (0, n9_node_log_1.default)(appName, {
            level: environment === 'development' ? 'debug' : undefined,
            ...options.logOptions,
            formatJSON: options.enableLogFormatJSON,
        });
    }
}
function applyShutdownOptionsDefaults(options) {
    options.shutdown = options.shutdown || {};
    options.shutdown.enableGracefulShutdown =
        typeof options.shutdown.enableGracefulShutdown === 'boolean'
            ? options.shutdown.enableGracefulShutdown
            : true;
    options.shutdown.timeout =
        typeof options.shutdown.timeout === 'number' ? options.shutdown.timeout : 25 * 1000;
    options.shutdown.waitDurationBeforeStop =
        typeof options.shutdown.waitDurationBeforeStop === 'number'
            ? options.shutdown.waitDurationBeforeStop
            : 10000;
}
function applyOpenApiOptionsDefaults(options, environment) {
    if (!options.openapi) {
        options.openapi = {};
    }
    options.openapi.isEnabled =
        typeof options.openapi.isEnabled === 'boolean' ? options.openapi.isEnabled : true;
    options.openapi.jsonUrl = options.openapi.jsonUrl || '/documentation.json';
    options.openapi.swaggerui = options.openapi.swaggerui || {
        ...options.openapi.swaggerui,
        swaggerUrl: '../documentation.json',
    };
    options.openapi.generateDocumentationOnTheFly =
        typeof options.openapi.generateDocumentationOnTheFly === 'boolean'
            ? options.openapi.generateDocumentationOnTheFly
            : environment === 'development';
}
function applyHttpOptionsDefaults(options) {
    // Defaults options for routing-controller
    const defaultRoutingControllerOptions = {
        defaults: {
            // with this option, null will return 404 by default
            nullResultCode: 404,
            // with this option, void or Promise<void> will return 204 by default
            undefinedResultCode: 204,
        },
        defaultErrorHandler: false,
        controllers: [`${options.path}/**/*.controller.*s`],
        plainToClassTransformOptions: {
            exposeDefaultValues: true,
        },
    };
    // Configure morgan
    morgan.token('total-response-time', (req, res, digits = 3) => {
        if (!req._startAt) {
            // missing request start time
            return;
        }
        // // calculate diff
        const reqStartTime = req._startAt;
        const resEndTime = process.hrtime();
        const ms = (resEndTime[0] - reqStartTime[0]) * 1e3 + (resEndTime[1] - reqStartTime[1]) * 1e-6;
        // return truncated value
        return ms.toFixed(digits);
    });
    // Default options
    options.http = options.http || {};
    options.http.port = options.http.port || process.env.PORT || 5000;
    options.http.logLevel =
        typeof options.http.logLevel !== 'undefined'
            ? options.http.logLevel
            : (tokens, req, res) => {
                const formatLogInJSON = options.log.formatJSON;
                if (formatLogInJSON) {
                    return JSON.stringify({
                        'method': tokens.method(req, res),
                        // eslint-disable-next-line @typescript-eslint/naming-convention
                        'request-id': options.enableRequestId
                            ? `(${req.headers['x-request-id']})`
                            : '',
                        'path': tokens.url(req, res),
                        'status': tokens.status(req, res),
                        'durationMs': Number.parseFloat(tokens['response-time'](req, res)),
                        'totalDurationMs': Number.parseFloat(tokens['total-response-time'](req, res)),
                        // eslint-disable-next-line @typescript-eslint/naming-convention
                        'content-length': tokens.res(req, res, 'content-length'),
                    });
                }
                return [
                    tokens.method(req, res),
                    tokens.url(req, res),
                    tokens.status(req, res),
                    tokens['response-time'](req, res),
                    'ms - ',
                    tokens.res(req, res, 'content-length'),
                ].join(' ');
            };
    options.http.routingController = {
        ...defaultRoutingControllerOptions,
        ...options.http.routingController,
    };
    options.http.routingController.interceptors = [session_loader_interceptor_1.SessionLoaderInterceptor, error_handler_interceptor_1.ErrorHandler];
    if (options.sentry) {
        options.http.routingController.interceptors.push(sentry_request_interceptor_1.SentryRequestInterceptor);
        if (options.sentry.additionalIntegrations?.find((additionalIntegration) => additionalIntegration.kind === 'tracing')) {
            options.log.module('sentry').info('Sentry tracing enabled');
            options.http.routingController.interceptors.push(sentry_tracing_interceptor_1.SentryTracingInterceptor);
        }
    }
    if (options.prometheus.isEnabled) {
        options.http.routingController.interceptors.push(prometheus_interceptor_1.PrometheusInterceptor);
    }
    options.http.routingController.authorizationChecker = async (action) => {
        if (!action.request.headers.session) {
            throw new n9_node_utils_1.N9Error('session-required', 401);
        }
        try {
            action.request.session = JSON.parse(action.request.headers.session);
        }
        catch (err) {
            throw new n9_node_utils_1.N9Error('session-header-is-invalid', 401);
        }
        if (!action.request.session.userId) {
            throw new n9_node_utils_1.N9Error('session-header-has-no-userId', 401);
        }
        return true;
    };
    options.http.routingController.validation = {
        whitelist: true,
        forbidNonWhitelisted: true,
    };
    if (!options.http.bodyParser) {
        options.http.bodyParser = {
            limit: '1024kb',
        };
    }
}
function applyHttpClientOptionsDefaults(options, environment) {
    const httpClientConfOptions = options.httpClient ?? {};
    const gotOptions = {
        responseType: 'json',
        ...httpClientConfOptions.gotOptions,
    };
    const sensitiveHeadersOptions = {
        alterSensitiveHeaders: environment !== 'development',
        alteringFormat: /(?!^)[\s\S](?!$)/g,
        sensitiveHeaders: ['Authorization'],
        ...httpClientConfOptions.sensitiveHeadersOptions,
    };
    options.httpClient = {
        gotOptions,
        sensitiveHeadersOptions,
    };
}
// Prometheus metrics
function applyPrometheusOptionsDefault(options) {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-boolean-literal-compare
    if (options.prometheus?.isEnabled !== false) {
        options.prometheus = options.prometheus || {};
        options.prometheus.isEnabled = true;
        options.prometheus.port =
            typeof options.prometheus.port === 'number' ? options.prometheus.port : 9101;
    }
}
function applySentryOptionsDefault(options, environment) {
    if (process.env.SENTRY_DSN) {
        if (!options.sentry)
            options.sentry = {};
        if (!options.sentry.initOptions)
            options.sentry.initOptions = {};
        options.sentry.initOptions.dsn = process.env.SENTRY_DSN;
    }
    if (options.sentry) {
        if (!options.sentry.initOptions?.dsn) {
            throw new n9_node_utils_1.N9Error('missing-sentry-dsn', 400, { options });
        }
        if (!options.sentry.forceCustomOptions) {
            if (!options.sentry.initOptions.integrations) {
                options.sentry.initOptions.integrations = [new Sentry.Integrations.Http({ tracing: true })];
            }
            if (!options.sentry.initOptions.environment) {
                options.sentry.initOptions.environment = environment;
            }
            if (Utils.isNil(options.sentry.initOptions.tracesSampleRate)) {
                options.sentry.initOptions.tracesSampleRate = 1;
            }
            if (!options.sentry.additionalIntegrations) {
                options.sentry.additionalIntegrations = [
                    {
                        kind: 'tracing',
                    },
                ];
            }
        }
    }
}
function applyAPMOptionsDefault(options, environment, appName) {
    if (options.apm?.newRelicOptions?.licenseKey && !process.env.NEW_RELIC_LICENSE_KEY) {
        process.env.NEW_RELIC_LICENSE_KEY = options.apm.newRelicOptions.licenseKey;
    }
    if (process.env.NEW_RELIC_LICENSE_KEY && process.env.NEW_RELIC_ENABLED?.trim() !== 'false') {
        if (!options.apm)
            options.apm = {};
        options.apm.type = 'newRelic';
        if (!options.apm.newRelicOptions)
            options.apm.newRelicOptions = {};
        if (!options.apm.newRelicOptions.appName) {
            const env = process.env.NEW_RELIC_ENV_NAME || environment;
            options.apm.newRelicOptions.appName = [env, appName].join('-');
        }
        if (options.apm.newRelicOptions.appName && !process.env.NEW_RELIC_APP_NAME) {
            process.env.NEW_RELIC_APP_NAME = options.apm.newRelicOptions.appName;
        }
    }
}
function applyConfOptionsDefaults(options) {
    const defaultOptions = {
        validation: {
            isEnabled: false,
            formatValidationErrors: true,
            formatWhitelistErrors: true,
        },
    };
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-boolean-literal-compare
    if (options.conf?.validation?.classType && options.conf?.validation?.isEnabled !== false) {
        defaultOptions.validation.isEnabled = true;
    }
    options.conf = _.merge(defaultOptions, options.conf);
}
exports.applyConfOptionsDefaults = applyConfOptionsDefaults;
function getModulesPath(options) {
    if (options.path) {
        if (fs.existsSync(options.path))
            return options.path;
    }
    else {
        const appRootDirPath = appRootDir.get();
        const modulesInSrcPath = (0, path_1.join)(appRootDirPath, 'src', 'modules');
        if (fs.existsSync(modulesInSrcPath))
            return modulesInSrcPath;
        const modulesInDistPath = (0, path_1.join)(appRootDirPath, 'dist', 'modules');
        if (fs.existsSync(modulesInDistPath))
            return modulesInDistPath;
        const modulesInSameFolderPath = (0, path_1.join)(appRootDirPath, 'modules');
        if (fs.existsSync(modulesInSameFolderPath))
            return modulesInSameFolderPath;
    }
    throw new n9_node_utils_1.N9Error('modules-path-not-found', 400, { options });
}
exports.getModulesPath = getModulesPath;
function applyDefaultValuesOnOptions(options, environment, appName) {
    options.path = getModulesPath(options);
    options.hasProxy = typeof options.hasProxy === 'boolean' ? options.hasProxy : true;
    options.enableRequestId =
        typeof options.enableRequestId === 'boolean' ? options.enableRequestId : true;
    applyConfOptionsDefaults(options);
    applyLogsOptionsDefaults(options, appName, environment);
    applyShutdownOptionsDefaults(options);
    applyPrometheusOptionsDefault(options);
    applySentryOptionsDefault(options, environment);
    applyAPMOptionsDefault(options, environment, appName);
    applyOpenApiOptionsDefaults(options, environment);
    applyHttpOptionsDefaults(options);
    applyHttpClientOptionsDefaults(options, environment);
}
exports.applyDefaultValuesOnOptions = applyDefaultValuesOnOptions;
function mergeOptionsAndConf(options, optionsInConf) {
    return _.merge(options, optionsInConf);
}
exports.mergeOptionsAndConf = mergeOptionsAndConf;
function getLoadingConfOptions(options) {
    const defaultLoadingConfOptions = {
        path: (0, path_1.join)(getModulesPath(options), '..', 'conf'),
    };
    return _.merge(defaultLoadingConfOptions, options.conf?.n9NodeConf);
}
exports.getLoadingConfOptions = getLoadingConfOptions;
//# sourceMappingURL=options.js.map