"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.initExposedConf = exports.onShutdownAsked = void 0;
const n9_node_utils_1 = require("@neo9/n9-node-utils");
const express_1 = require("@promster/express");
const fs = require("fs");
const SwaggerUi = require("swagger-ui-express");
const generate_documentation_json_1 = require("./generate-documentation-json");
const RoutesService = require("./routes.service");
let shutdownAsked = false;
let exposedConf;
function onShutdownAsked() {
    shutdownAsked = true;
}
exports.onShutdownAsked = onShutdownAsked;
function initExposedConf(conf) {
    exposedConf = conf;
}
exports.initExposedConf = initExposedConf;
function init(expressApp, options, packageJson, env, logger) {
    expressApp.get('/', (req, res, next) => {
        res.status(200).json({ name: packageJson.name });
        next();
    });
    // Monitoring route
    expressApp.get('/ping', async (req, res, next) => {
        if (shutdownAsked) {
            res.status(503).json({ response: 'server-shutting-down' });
            (0, express_1.signalIsNotUp)();
            next();
            return;
        }
        if (options.http.ping?.dbs) {
            for (const db of options.http.ping.dbs) {
                // eslint-disable-next-line @typescript-eslint/no-invalid-this
                if (!(await db.isConnected.bind(db.thisArg || this)())) {
                    logger.error(`[PING] Can't connect to DB : ${db.name}`);
                    res.status(500).json(new n9_node_utils_1.N9Error(`db-${db.name}-unreachable`, 500));
                    if (options.prometheus.isEnabled) {
                        (0, express_1.signalIsNotUp)();
                    }
                    next();
                    return;
                }
            }
            if (options.prometheus.isEnabled) {
                (0, express_1.signalIsUp)();
            }
            if (options.http.ping.dbs.length > 1) {
                res.status(200).json({ response: `pong-dbs-${options.http.ping.dbs.length}` });
            }
            else {
                res.status(200).json({ response: `pong-db` });
            }
        }
        else {
            res.status(200).json({ response: 'pong' });
        }
        next();
    });
    // Return app version
    expressApp.get('/version', (req, res, next) => {
        res.status(200).json({ version: packageJson.version });
        next();
    });
    expressApp.get('/conf', (req, res, next) => {
        res.status(200).json(exposedConf);
        next();
    });
    expressApp.get('/routes', (req, res, next) => {
        res.status(200).json(RoutesService.getRoutes());
        next();
    });
    if (options.openapi.isEnabled && env !== 'production') {
        expressApp.get(options.openapi.jsonUrl, (req, res) => {
            let spec;
            if (options.openapi.generateDocumentationOnTheFly) {
                spec = (0, generate_documentation_json_1.generateDocumentationJson)(options);
            }
            else {
                const documentationJsonPath = (0, generate_documentation_json_1.getDocumentationJsonPath)(options);
                if (fs.existsSync(documentationJsonPath)) {
                    spec = JSON.parse(fs.readFileSync(documentationJsonPath).toString());
                    options.log.debug(`Documentation fetched from file ${documentationJsonPath}`);
                }
                else {
                    options.log.error(`Generated documentation not found from file ${documentationJsonPath}`);
                    res.status(404).json(new n9_node_utils_1.N9Error('generated-documentation-not-found', 404));
                    return;
                }
            }
            res.header('Access-Control-Allow-Origin', '*');
            res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept');
            res.json(spec);
        });
        expressApp.use('/documentation', SwaggerUi.serve, SwaggerUi.setup(null, options.openapi.swaggerui));
    }
    // Handle 404 errors
    expressApp.use((req, res, next) => {
        if (!res.headersSent) {
            const err = new n9_node_utils_1.N9Error('not-found', 404, { url: req.url });
            options.log.warn(err.message, { err });
            let error;
            if (!expressApp.get('env') || ['development', 'test'].includes(expressApp.get('env'))) {
                error = err;
            }
            return res.status(404).json({
                error,
                code: err.message,
                status: err.status,
                context: err.context,
            });
        }
        next();
    });
}
exports.init = init;
//# sourceMappingURL=routes.js.map