export declare class Cargo<RESPONSE extends object = {
    _id: string;
}, REQUEST extends object | string = string> {
    private cargoType;
    private workerFn;
    private readonly dispatchFn?;
    private throwOnEmptyValue;
    private cargoQueue;
    /**
     * Return a new cargo wich can be use to automatically batch tasks executions.
     * This cargo rely on the caolan/async cargoQueue.
     * Documentation can be found here https://caolan.github.io/async/v3/docs.html#cargoQueue
     * This cargo only add the dispatch logic to return the unitary response
     * in the Promise resolution from the batched responses provided by the workers
     *
     * Every time a request is added if a worker is available it will handle the request
     * if no worker is available the request will be kept to be handled by the next available worker.
     * This allow minimal overhead when the request frequency is low, and allow to batch more request
     * when the request rate increases.
     *
     * The add method take a REQUEST as parameter and returns a Promise that will be resolved with a value of type RESPONSE.
     * The workerFn must handle a batch of REQUEST type. After the workerFn returns, the response will be forward to the dispatch
     * method to match requests with corresponding values.
     * The dispatchFn must find for a REQUEST the corresponding values in the workerFn response. it returns a value of type RESPONSE.
     *
     * REQUEST and RESPONSE can be objects,
     *
     * @param cargoType : string used for logging purpose
     * @param workerFn asynchronous function : used to handle a bask of requests. The result will be forward to the dispatch function
     * @param dispatchFn synchronous function : that maps batched response values to unitary requests <br />
     * dispatch function accept a request, all response and give the corresponding response for a request <br />
     * default : map using _id property
     * @param throwOnEmptyValue boolean : if true an exception will be throw if not value can be dispatched for a request
     * @param nbMaxConcurrentWorkers number : default 2 : max number of active worker
     * @param taskPerWorker number : default 200 : max number of request that a worker can handle in one batch
     */
    constructor(cargoType: string, workerFn: (requests: REQUEST[]) => Promise<any>, dispatchFn?: (request: REQUEST, responses: any) => RESPONSE, throwOnEmptyValue?: boolean, nbMaxConcurrentWorkers?: number, taskPerWorker?: number);
    get(request: REQUEST): Promise<RESPONSE>;
}
