"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.N9HttpClient = void 0;
const n9_node_utils_1 = require("@neo9/n9-node-utils");
const cls_hooked_1 = require("cls-hooked");
const fast_safe_stringify_1 = require("fast-safe-stringify");
const got_1 = require("got");
const nanoid_1 = require("nanoid");
const QueryString = require("query-string");
const stream_1 = require("stream");
const urlJoin = require("url-join");
const requestid_1 = require("../requestid");
const utils_1 = require("../utils");
class N9HttpClient {
    static getUriFromUrlParts(url) {
        let uri;
        if (Array.isArray(url))
            uri = urlJoin(...url);
        else
            uri = urlJoin(url);
        return uri;
    }
    static prepareErrorCodeAndStatus(e) {
        let code;
        if (typeof e.response?.body === 'string') {
            code = e.response?.body.substring(0, 500);
        }
        else {
            try {
                const errorJson = typeof e.response?.body === 'object' ? e.response?.body : JSON.parse(e.response?.body);
                code = errorJson?.code ?? errorJson?.message;
            }
            catch (error) {
                code = e.code;
            }
        }
        if (!code)
            code = e.code;
        const status = e.response?.statusCode;
        return { code, status };
    }
    static alterHeaders(headers, sensitiveHeadersOptions) {
        if (headers &&
            sensitiveHeadersOptions.alterSensitiveHeaders &&
            Object.keys(headers).length > 0) {
            for (const header of sensitiveHeadersOptions.sensitiveHeaders) {
                if (!headers[header])
                    continue;
                const rawHeader = headers[header];
                headers[header] = rawHeader.replace(sensitiveHeadersOptions.alteringFormat, '*');
            }
        }
        return headers;
    }
    constructor(logger, options, maxBodyLengthToLogError = 100) {
        this.logger = logger;
        this.maxBodyLengthToLogError = maxBodyLengthToLogError;
        this.baseClientOptions = this.buildHttpClientOptionsWithDefaults(logger, options);
    }
    /**
     * N9HttpClientQueryParams samples : https://github.com/request/request/blob/master/tests/test-qs.js
     */
    async get(url, queryParams, headers, options) {
        return this.request('get', url, queryParams, headers, undefined, options);
    }
    async post(url, body, queryParams, headers, options) {
        return this.request('post', url, queryParams, headers, body, options);
    }
    async put(url, body, queryParams, headers, options) {
        return this.request('put', url, queryParams, headers, body, options);
    }
    async delete(url, body, queryParams, headers, options) {
        return this.request('delete', url, queryParams, headers, body, options);
    }
    async options(url, queryParams, headers, options) {
        return this.request('options', url, queryParams, headers, undefined, options);
    }
    async patch(url, body, queryParams, headers, options) {
        return this.request('patch', url, queryParams, headers, body, options);
    }
    async head(url, queryParams, headers, options) {
        return this.request('head', url, queryParams, headers, undefined, options);
    }
    async request(method, url, queryParams, headers = this.baseClientOptions.gotOptions.headers, body, options = {}) {
        const uri = N9HttpClient.getUriFromUrlParts(url);
        const namespaceRequestId = (0, cls_hooked_1.getNamespace)(requestid_1.RequestIdNamespaceName);
        const requestId = namespaceRequestId?.get(requestid_1.RequestIdKey) || (0, nanoid_1.nanoid)(10);
        // eslint-disable-next-line @typescript-eslint/naming-convention
        const sentHeaders = { ...headers, 'x-request-id': requestId };
        const searchParams = typeof queryParams === 'string'
            ? queryParams
            : QueryString.stringify(queryParams, { arrayFormat: 'none' });
        const startTime = Date.now();
        const optionsSent = {
            ...this.baseClientOptions.gotOptions,
            method,
            searchParams,
            headers: sentHeaders,
            json: body,
            ...options,
        };
        try {
            const res = await (0, got_1.default)(uri, optionsSent);
            // for responses 204
            if (optionsSent.responseType === 'json' && res.body === '')
                return;
            return res.body;
        }
        catch (e) {
            const durationMs = Date.now() - startTime;
            const bodyJSON = (0, fast_safe_stringify_1.default)(body);
            const { code, status } = N9HttpClient.prepareErrorCodeAndStatus(e);
            this.logger.error(`Error on [${method} ${uri}] ${e.message}`, {
                uri,
                method,
                status,
                durationMs,
            });
            const alteredHeaders = N9HttpClient.alterHeaders(optionsSent.headers, this.baseClientOptions.sensitiveHeadersOptions);
            throw new n9_node_utils_1.N9Error(code.toString(), status, {
                uri,
                method,
                queryParams,
                durationMs,
                code: e.code ?? e.message,
                body: body && bodyJSON.length < this.maxBodyLengthToLogError ? bodyJSON : undefined,
                headers: alteredHeaders,
                srcError: e.response?.body ?? e,
            });
        }
    }
    async raw(url, options) {
        const uri = N9HttpClient.getUriFromUrlParts(url);
        const startTime = Date.now();
        const optionsSent = {
            ...this.baseClientOptions.gotOptions,
            ...options,
        };
        try {
            const res = await (0, got_1.default)(uri, optionsSent);
            // for responses 204
            if (optionsSent.responseType === 'json' && res.body === '')
                return;
            return res.body;
        }
        catch (e) {
            const durationMs = Date.now() - startTime;
            const { code, status } = N9HttpClient.prepareErrorCodeAndStatus(e);
            this.logger.error(`Error on [${options.method} ${uri}]`, {
                status,
                durationMs,
            });
            const alteredHeaders = N9HttpClient.alterHeaders(optionsSent.headers, this.baseClientOptions.sensitiveHeadersOptions);
            throw new n9_node_utils_1.N9Error(code.toString(), status, {
                uri,
                options: (0, fast_safe_stringify_1.default)({ ...optionsSent, headers: alteredHeaders }),
                error: e,
                ...e.context,
            });
        }
    }
    async requestStream(url, 
    // issue https://github.com/sindresorhus/got/issues/954#issuecomment-579468831
    options) {
        const responseAsStream = new stream_1.PassThrough();
        const startTime = Date.now();
        const uri = N9HttpClient.getUriFromUrlParts(url);
        const requestResponse = got_1.default.stream(uri, options);
        requestResponse.pipe(responseAsStream);
        let incomingMessage;
        let durationMsTTFB;
        try {
            incomingMessage = await new Promise((resolve, reject) => {
                requestResponse.on('error', (err) => reject(err));
                requestResponse.on('response', (response) => {
                    response.on('end', () => {
                        const durationMsTTLB = Date.now() - startTime;
                        if (durationMsTTFB !== null && durationMsTTFB !== undefined) {
                            const durationDLMs = durationMsTTLB - durationMsTTFB;
                            this.logger.debug(`File TTLB : ${durationMsTTLB} ms TTDL : ${durationDLMs} ms`, {
                                durationDLMs,
                                url,
                                durationMs: durationMsTTLB,
                            });
                        }
                        else {
                            this.logger.debug(`File TTLB : ${durationMsTTLB} ms`, {
                                url,
                                durationMs: durationMsTTLB,
                            });
                        }
                    });
                    if (response.statusCode >= 400) {
                        reject(response);
                    }
                    resolve(response);
                });
            });
        }
        catch (e) {
            const durationMs = Date.now() - startTime;
            this.logger.error(`Error on [${options?.method ?? 'GET'} ${uri}]`, {
                status: e.statusCode,
                durationMs,
            });
            this.logger.debug(`File TTFB : ${durationMs} ms`, {
                url,
                durationMs,
                statusCode: e.statusCode,
            });
            const { code, status } = N9HttpClient.prepareErrorCodeAndStatus(e);
            const alteredHeaders = N9HttpClient.alterHeaders(options?.headers, this.baseClientOptions.sensitiveHeadersOptions);
            throw new n9_node_utils_1.N9Error(code?.toString() || 'unknown-error', status, {
                uri,
                method: options?.method,
                code: e.code || code,
                headers: alteredHeaders,
                srcError: e,
                responseTime: durationMs,
            });
        }
        durationMsTTFB = Date.now() - startTime;
        this.logger.debug(`File TTFB : ${durationMsTTFB} ms`, {
            url,
            durationMs: durationMsTTFB,
            statusCode: incomingMessage.statusCode,
        });
        return { incomingMessage, responseAsStream };
    }
    buildHttpClientOptionsWithDefaults(logger, clientOptions) {
        const environment = (0, utils_1.getEnvironment)();
        const gotOptions = {
            responseType: 'json',
            resolveBodyOnly: false,
            hooks: {
                beforeRetry: [
                    (options, error, retryCount) => {
                        let level;
                        if (error?.response?.statusCode && error.response.statusCode < 500) {
                            level = 'info';
                        }
                        else {
                            level = 'warn';
                        }
                        if (logger.isLevelEnabled(level)) {
                            logger[level](`Retry call [${options.method} ${options.url?.toString()}] n°${retryCount} due to ${error?.code ?? error?.name} ${error?.message}`, {
                                errString: (0, fast_safe_stringify_1.default)(error),
                                status: error?.response?.statusCode,
                            });
                        }
                    },
                ],
            },
            ...clientOptions?.gotOptions,
        };
        const sensitiveHeadersOptions = {
            alterSensitiveHeaders: environment !== 'development',
            sensitiveHeaders: ['Authorization'],
            alteringFormat: /(?!^)[\s\S](?!$)/g,
            ...clientOptions?.sensitiveHeadersOptions,
        };
        return { gotOptions, sensitiveHeadersOptions };
    }
}
exports.N9HttpClient = N9HttpClient;
//# sourceMappingURL=http-client-base.js.map